<?php
/* File:	mark_page.php
 * Summary:  	insert a marker in a page to be recorded by bbclone
 * Description:
 * 
 *  mark_page.php uses 10 files to store the data, to increase speed
 *  and avoid locking problems
 *
 *  Usage : Insert this code in the file including the counter :
 *  <?php
 *  DEFINE ("_BBCLONE_DIR", "mybbclone/");
 *  DEFINE ("COUNTER", _BBCLONE_DIR . "mark_page.php");
 *  if (file_exists(COUNTER)) include (COUNTER);
 *  ?>
 *
 *  See the INSTALL file for further explanations.
 * 
 * License:
 * 
 * This file is part of BBclone (A PHP web counter on steroids)
 *
 * $Header: /cvsroot/bbclone/bbclone/mark_page.php,v 1.11 2002/09/04 20:38:35 jdurden Exp $
 *
 * Copyright (C) 2001-2002, the BBclone Team (see the file AUTHORS
 * distributed with this library)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

// Checking if BBCLONE_DIR is set
if (!defined("_BBCLONE_DIR"))
{ 
  echo "\n<!-- BBclone Error: You have to correctly defined _BBCLONE_DIR -->\n";  
  return;
}  
else
{
  global $BBCLONE_DIR;
  $BBCLONE_DIR = ereg("/$", _BBCLONE_DIR) ? _BBCLONE_DIR : _BBCLONE_DIR . "/";
}

// Prevent to load twice mark_page.php
if (defined("_BBC_PAGE_MARKED"))
{
  return;
}
else
{
  define("_BBC_PAGE_MARKED",1);
}

// Loading all the desired libraries
require($BBCLONE_DIR . "constants.php");
require($GLOBALS["BBC_LIB_PATH"] . "io.php");
//require($GLOBALS["BBC_CONF_PATH"] . $GLOBALS["BBC_CONFIG_FILE"]);


// Main Class Counter
class marker
{
  var $filename;
  
  // Constructor
  function counter()
    {
      mt_srand((double)microtime()*1000000);
    }

  // Choose the counterfile to use
  function choose_counterfile($fileprefix, $filesuffix)
    {
      $salt = mt_rand(0,9);
      return ($fileprefix.$salt.$filesuffix);
    }

  // Converts quad dotted IPv4 addresses to Hexa
  function convert_ip($ipaddr)
    {
      $dotted = preg_split( "/[.]+/", $ipaddr);
      $ip = (double)0;
      $y = 0x1000000;
      
      for( $i=0; $i<4; $i++)
	{
	  $ip += ($dotted[$i] * ($y));
	  $y = ($y >> 8);
	}
      return $ip;
    }

  // Open the counter file and locks it
  function open_counterfile()
    {
      $filename = $this->choose_counterfile ( $GLOBALS["BBC_CACHE_PATH"] . $GLOBALS["BBC_COUNTER_PREFIX"],
					      $GLOBALS["BBC_COUNTER_SUFFIX"] );
      $counterp = fopenlock($filename,"a+"); 

      if (!$counterp)
	{
	  echo "<!-- mark_page.php: unable to use \"$filename\" -->\n";
	  return false;
	}

      return $counterp;
    }

  // Sets the values for the string
  function set_values()
    {
      $localtime = time(); // UNIX timestamp

      global $HTTP_X_FORWARDED_FOR, $HTTP_CLIENT_IP, 
	$HTTP_FORWARDED, $HTTP_SP_HOST, 
	$REMOTE_ADDR, $REMOTE_HOST, 
	$HTTP_USER_AGENT, $HTTP_REFERER, $REQUEST_URI;

      // Through the looking-glass, quick and efficient :)
      $PROXY = array ("$HTTP_X_FORWARDED_FOR", "$HTTP_CLIENT_IP", "$HTTP_FORWARDED", "$HTTP_SP_HOST");

      for($k = 0, $nbproxy = sizeof($PROXY); $k < $nbproxy; $k++)
	{
	  if ($PROXY[$k])
	    {
	      $iptest = gethostbyname($PROXY[$k]);

	      //Sort everything out that hasn't got IPv4 format
	      if (!eregi("^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})$", $iptest, $regs))
		{
		  $ip = $REMOTE_ADDR;
		  $hexip = $this->convert_ip($ip);
		}
	      else
		{
		  //Sort out local and private addresses
		  if (((int) $regs[1] == 0) || ((int) $regs[1] == 10) || ((int) $regs[1] == 127) ||
		     (((int) $regs[1] == 172) && ((int) $regs[2] > 15) && ((int) $regs[2] < 32)) ||
		     (((int) $regs[1] == 192) && ((int) $regs[2] == 168)))
		    {
		      $ip = $REMOTE_ADDR;
		      $hexip = $this->convert_ip($ip);
		    }
		  //Sort out multicast and addresses with values out of range
		  else if (((int) $regs[1] > 223) || ((int) $regs[2] > 255) ||
			   ((int) $regs[3] > 255) || ((int) $regs[4] == 0)  ||
			   ((int) $regs[4] > 254))
		    {
		      $ip = $REMOTE_ADDR;
		      $hexip = $this->convert_ip($ip);
		    }
		  //Take the addresses remained and assume they're the real ones
		  else
		    {
		      $ip = $iptest;
		      $hexip = $this->convert_ip($ip);
		    }
		}
	      // The following lines were added to avoid invalid hostnames in the logfiles. Now
	      // everything that can't be ressolved back will stay an IP address.
	      $temphost = gethostbyaddr($ip);
	      $tempip = gethostbyname($temphost);
	      if ($ip != "$tempip") {
		$ev_hostname = $ip;
	      }
	      else {
		$ev_hostname = $temphost;
		break;
	      }
	    }
	}
      if (!$ip)
	{
	  if ($REMOTE_ADDR)
	    // Once again we need to avoid invalid hostnames in the logfiles.
	    {
	      $ip = $REMOTE_ADDR;
	      $hexip = $this->convert_ip($ip);
	      $temphost = gethostbyaddr($ip);
	      $tempip = gethostbyname($temphost);
	      if ($ip != "$tempip") {
		$ev_hostname = $ip;
	      }
	      else {
		$ev_hostname = $temphost;
	      }
	    }
	  else
	    {
	      $ev_hostname = "unknown";
	      $ip = "unknown";
	      $hexip = "unknown";
	    }
	}

      if (!$HTTP_USER_AGENT) $HTTP_USER_AGENT = "unknown";
      if (!$HTTP_REFERER) $HTTP_REFERER = "unknown";

      $str = 
	strip_tags( "$localtime|$hexip|$ip|$ev_hostname|$REMOTE_HOST|$HTTP_USER_AGENT|".
		    "$HTTP_REFERER|$REQUEST_URI|" . 
		    (defined("_BBC_PAGE_NAME") ? _BBC_PAGE_NAME : "" ));
      return $str;
    }

  // Outputs some values to a file
  function write_values($dest,$string)
    {
      return(fputs($dest, $string."\n"));
    }

  // Close the counterfile
  function close_counterfile($filep)
    {
      flush();
      return(flock($filep, 3) && fclose($filep));
    }

  // err... the main stuff	
  function main()
    {
      $mystring = $this->set_values();
      $file = $this->open_counterfile();

      if ($file)
	{
	  return (($this->write_values($file,$mystring)) && ($this->close_counterfile($file)));
	}
    }
}

// Launching this class
$mark = new marker;

if(!$mark->main()) {
  echo ("\n<!-- BBClone v ". $GLOBALS["BBC_VERSION"] ." KO -->\n");
}
else {
  echo ("\n<!-- BBClone v ". $GLOBALS["BBC_VERSION"] ." OK -->\n");
}

?>
